"""
app.py

A common utility app for flask

Contain common view interface

-. LoginManager
-. UserManager
-. Error
"""

import os
import functools
import jinja2
import redis
from utils import is_valid_email, is_valid_password, hash_string, \
    verify_hash_string, generate_random_string, to_currency, to_slug, \
    add_path_to_jinja
from flask import abort
from flask.ext.login import current_user
from flask_kvsession import KVSessionExtension
from simplekv.memory.redisstore import RedisStore
import humanize
import mistune

# ------------------------------------------------------------------------------

__version__ = "0.22.0"



def get_redis(dsn):
    """
    Return the redis connection
    :param dsn: The dsn url
    :return: Redis
    """
    return redis.StrictRedis.from_url(url=dsn)

def with_user_roles(*roles):
    """
    with_user_roles(*roles)

    It allows to check if a user has access to a view by adding the decorator
    accept_roles(*roles)

    Using accept_user_roles() requires flask-login.

    In your model, you must have a property 'role', which will be invoked to
    be compared to the roles provided.

    If current_user doesn't have a role, it will throw a 403

    If the current_user is not logged in will throw a 401

    * Require Flask-Login
    ---
    Usage

    @app.route('/user')
    @login_require
    @with_user_roles('admin', 'user')
    def user_page(self):
        return "You've got permission to access this page."
    """
    def wrapper(f):
        @functools.wraps(f)
        def wrapped(*args, **kwargs):
            if current_user.is_authenticated():
                if not hasattr(current_user, "role"):
                    raise AttributeError("<'role'> doesn't exist in login 'current_user'")
                if current_user.role not in roles:
                    return abort(403)
            else:
                return abort(401)
            return f(*args, **kwargs)
        return wrapped
    return wrapper


class AdminRoleMixin(object):
    pass

class AppError(Exception):
    """ For exception in application pages """
    pass


class Session(object):
    """ Set the Session. Using KVSession """
    def __init__(self, app):
        _redis = get_redis(app.config.get("KVSESSION_REDIS_DSN"))
        store = RedisStore(_redis)
        KVSessionExtension(store, app)

# ------------------------------------------------------------------------------
# FILTERS

def to_date(dt, format="%m/%d/%Y"):
    return dt.strftime(format)

def strip_decimal(amount):
    return amount.split(".")[0]

def bool_to_yes(b):
    return "Yes" if b is True else "No"

def bool_to_int(b):
    return 1 if b is True else 0


jinja2.filters.FILTERS.update({
    "to_currency": to_currency,
    "strip_decimal": strip_decimal,
    "to_date": to_date,
    "to_int": int,
    "to_slug": to_slug,
    "to_intcomma": humanize.intcomma,
    "to_intword": humanize.intword,
    "to_naturalday": humanize.naturalday,
    "to_naturaldate": humanize.naturaldate,
    "to_naturaltime": humanize.naturaltime,
    "markdown": mistune.markdown,
    "bool_to_yes": bool_to_yes,
    "bool_to_int": bool_to_int
})
